%clear all;
tic;

ReadSeaStates();

%Makes sure that the Hs list rounds to the nearest half value
Hs=round(Hs_G(:)*2)/2;

% Set number of states in the markov chain
% 
% Beware of setting this too high. If there are too many states, some of
% the states will be absorbing, that is, P(j,j) = 1, which means it can
% never transition to other states.

numStates = 7;

% Find upper limit for Hs values and divide the values into even bins
ul = max(Hs);
% Find state ranges - first state [0,stateRange] and so on
stateRange = ul / numStates;
% State values - stateRange, 2xstateRange and so on up til ul
stateValues = stateRange:stateRange:ul;
% Initialize 1D-matrix holding the state of each data point
HsState = zeros(length(Hs),1);

% Find each data point's state
for i = 1:length(Hs)
    % For each data point
    for j = 1:numStates
        % For each state
        if Hs(i) <= stateValues(j)
            % Data point is in state j
            HsState(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
end

% Find transitions
transitions = zeros(numStates);
for t = 1:length(HsState)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
    transitions(HsState(t),HsState(t+1)) = transitions(HsState(t),HsState(t+1)) + 1;
end

P_G = transitions;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates
    P_G(i,:) = P_G(i,:) / sum( P_G(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate = zeros(numStates);
for i = 1:numStates
    for j = 1:numStates
        if P_G(i,j) == 1
            absorbstate(i,j) = absorbstate(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate)) >= 1
    error('Absorbing states. Stopping. Consider reducing number of states or check data.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications = 100000;

% Random number seed
rng(Seed(q));

% Set starting state - should sample randomly
state = randi(numStates);

states = zeros(numReplications,1);

for i = 1:numReplications
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates
        prob = 0;
        % Accumulate probabilities
        for k = 1:j
            prob = prob + P_G(state,k);
        end
        
        if r <= prob
            % New state is found, j
            state = j;
            
            % Store the state we transition to
            states(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end

% If needed, Hs can be compared directly to the simulated results
simValues = zeros(numReplications,1);
for i = 1:numReplications
    simValues(i) = (states(i) * stateRange) - stateRange/2;
end


% Plot the distribution for the original data points and the simulated sea
% states. The number of samples won't correlate, but the general shape
% should correlate somewhat.
figure(1);
hist(Hs,numStates);
title('Sea States');
figure(2);
hist(simValues,numStates);
title('Simulation results');


% To see how the timeseries looks we can run
% figure(3);
% plot(simValues(1:50));


% Write the probabilities to the Markov file.
%filename = 'MarkovRes.xlsx';
%xlswrite(filename,Liste);

toc;